/*
 * Lua - libsensors wrapper
 */

#include <sensors/error.h>
#include <sensors/sensors.h>

#include <lua.h>
#include <lualib.h>
#include <lauxlib.h>

#define API4 (SENSORS_API_VERSION >= 0x400)

#define DEF_C(c) \
  {#c, SENSORS_ERR_##c}

#define DEF_F(f) \
  {#f, Lsensors_##f}

typedef struct const_reg {
  char *key;
  int value;
} const_reg;

static const const_reg C[] = {
  DEF_C(WILDCARDS),
  DEF_C(NO_ENTRY),
#if API4
  DEF_C(KERNEL),
  DEF_C(IO),
#else
  DEF_C(ACCESS),
  DEF_C(PROC),
#endif
  DEF_C(DIV_ZERO),
  DEF_C(CHIP_NAME),
  DEF_C(BUS_NAME),
  DEF_C(PARSE),
  DEF_C(ACCESS_W),
  DEF_C(ACCESS_R),
  {NULL, 0}
};

static int
Lsensors_init(lua_State *L) {
  FILE **f = (FILE **)luaL_checkudata(L, 1, LUA_FILEHANDLE);
  if(*f == NULL) luaL_error(L, "attempt to use a closed file");
  lua_pushinteger(L, sensors_init(*f));
  return 1;
}

static int
Lsensors_cleanup(lua_State *L) {
  sensors_cleanup();
  return 0;
}

static int
Lsensors_strerror(lua_State *L) {
  int i = luaL_checkint(L, 1);
  lua_pushstring(L, sensors_strerror(i));
  return 1;
}

static int
Lsensors_scan_all_chips(lua_State *L) {
  const sensors_chip_name *scn;
#if API4
  const sensors_feature *sf;
  const sensors_subfeature *ss;
#else
  const sensors_feature_data *sfd;
#endif
  double r;
  int n,n1,n2;
  char s[80];
  lua_newtable(L);
#if API4
  for(n = 0; (scn = sensors_get_detected_chips(NULL, &n)) != NULL; ) {
    sprintf(s, "%s-%x-%x", scn->prefix, scn->bus.nr, scn->addr);
#else
  for(n = 0; (scn = sensors_get_detected_chips(&n)) != NULL; ) {
    sprintf(s, "%s-%x-%x", scn->prefix, scn->bus, scn->addr);
#endif
    lua_pushstring(L, s);
    lua_newtable(L);
#if API4
    for(n1 = 0; (sf = sensors_get_features(scn, &n1)) != NULL; ) {
      for(n2 = 0; (ss = sensors_get_all_subfeatures(scn, sf, &n2)) != NULL; ) {
        if(sensors_get_value(scn, ss->number, &r) == 0) {
          lua_pushstring(L, ss->name);
#else
    for(n1 = n2 = 0; (sfd = sensors_get_all_features(scn, &n1, &n2)) != NULL; ) {
        if(sensors_get_feature(scn, sfd->number, &r) == 0) {
          lua_pushstring(L, sfd->name);
#endif
          lua_pushnumber(L, r);
          lua_rawset(L, -3);
        }
#if API4
      }
#endif
    }
    lua_rawset(L, -3);
  }
  return 1;
}

static const luaL_reg R[] = {
  DEF_F(init),
  DEF_F(cleanup),
  DEF_F(strerror),
  DEF_F(scan_all_chips),
  {NULL, NULL}
};

static void
reg_const(lua_State *L, const char *t, const const_reg *kv) {
  lua_pushstring(L ,t);
  lua_newtable(L);
  for(; kv->key != NULL; ++kv) {
    lua_pushstring(L, kv->key);
    lua_pushinteger(L, kv->value);
    lua_rawset(L, -3);
  }
  lua_rawset(L, -3);
}

LUALIB_API int
luaopen_libsensors(lua_State *L) {
  luaL_register(L, "sensors", R);
  reg_const(L, "ERR", C);
  return 1;
}

